package com.fri.http.legalchapter.rzfw.auth;

import com.fri.http.framework.config.YXDefaultServerConfig;
import com.fri.http.framework.logging.YXLogging;
import com.fri.http.legalchapter.rzfw.data.*;
import com.fri.http.legalchapter.rzfw.net.SocketSender;
import com.fri.http.legalchapter.rzfw.utils.Constans;
import com.fri.http.legalchapter.rzfw.utils.FormatBytesUtil;
import net.sf.json.JSONObject;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.logging.Logger;

/**
 * Created by GavinLi 李晓阳 on 2017/12/24.
 */
public class AuthApply {
    private Logger logger = YXLogging.getLogger(YXDefaultServerConfig.LOGGING_NAME_REQUEST_SERVER);

    /**
     * customerId 客户号 8字节 共用
     * terminalId 终端号 2字节 共用     1.0写死   2.0用应用名称 FormatBytesUtil.hexStringToBytes(preAuthBean.getAppName());
     * timeStamp 4字节时间戳
     * readCardControlsVersion 读卡控件版本 4字节
     * pictureControlsVersion 人像控件版本 4字节
     * certificateNumber 证书标识 or 业务站点
     * auAppData 身份认证申请数据区
     * version 协议版本 2字节 0x0001 共用
     * commandType 命令类型 1字节 0x00(请求)0x01(应答) 共用
     * command 命令 1字节 0x00(身份认证申请)0x01(身份认证) 共用但值不同
     * commandContent 命令内容 1字节 保留 0x00 共用
     * operation  操作 1字节 保留 0x00 共用
     * auAppSPData 身份认证 一包 Socket数据报文
     * applicationReturn 身份认证请求1.0 平台 响应结果
     * checkSignResult 身份认证请求1.0 平台 响应验签是否正确
     * date 返回时间戳
     * businessSerialNumberByte 流水号
     * randomBytesLength 随机数长度
     * randomData 随机数数据
     * errorDesc 错误描述
     */
    private byte[] customerId;
    private byte[] terminalId;
    private byte[] timeStamp;
    private byte[] readCardControlsVersion;
    private byte[] pictureControlsVersion;
    private byte[] certificateNumber;
    private byte[] auAppData;
    private byte[] version = new byte[]{0, 1};
    private byte[] commandType = new byte[]{0};
    private byte[] command = new byte[]{0};
    private byte[] commandContent = new byte[]{0};
    private byte[] operation = new byte[]{0};
    private byte[] auAppSPData;
    private byte[] applicationReturn;
    private boolean checkSignResult=false;
    private String date;
    private byte[] businessSerialNumberByte = null;
    private short randomBytesLength;
    private String randomData;
    private String errorDesc="未知错误";
    private byte[] customerId_Length=new byte[] { 1, 2, 3, 4, 5, 6, 7, 8 };
    private byte[] timeStamp_Length=new byte[] { 1, 2,3,4};
    private byte[] readCardControlsVersion_Length=new byte[] { 1, 2, 3, 4 };
    private byte[] pictureControlsVersion_Length=new byte[] { 1, 2, 3, 4 };
    PreAuthResponse preAuthResponse= new PreAuthResponse();
    public String authApplyExecute(String authapplyJson){
        JSONObject preAuthApply=JSONObject.fromObject(authapplyJson);
        PreAuthBean preAuthBean= (PreAuthBean) JSONObject.toBean(preAuthApply, PreAuthBean.class);

         /**
		 * 组装认证申请数据区 第一个包的数据区
		 */
        customerId= FormatBytesUtil.getBytesFromString(customerId_Length ,preAuthBean.getCustomerNumber());
        //TODO  终端号 2字节 共用     1.0写死   2.0用应用名称 FormatBytesUtil.hexStringToBytes(preAuthBean.getAppName());
        terminalId = Constans.terminalId;
        timeStamp =  FormatBytesUtil.getBytesFromString(timeStamp_Length, preAuthBean.getTimeStamp());
        readCardControlsVersion = FormatBytesUtil.getBytesFromString(readCardControlsVersion_Length,preAuthBean.getCardReaderVersion());
        pictureControlsVersion =FormatBytesUtil.getBytesFromString(pictureControlsVersion_Length,preAuthBean.getLiveDetectionControlVersion());
        certificateNumber=Constans.certificateNumber;

        PreAuthData auApp = new PreAuthData(customerId, terminalId,timeStamp,
                readCardControlsVersion, pictureControlsVersion, certificateNumber);
        auAppData = auApp.getPreAuthData();

        /**
		 * 组装认证申请Socket数据包 第一次UDP数据包
		 */

        SocketPackage auAppSP = new SocketPackage(version, commandType, command, commandContent, operation, auAppData);
        auAppSPData = auAppSP.getSocketPackage();
        applicationReturn = SocketSender.sendSocketPackage(auAppSPData);
        if(applicationReturn!=null){
            checkSignResult=DataSignature.signCheck(applicationReturn);
            System.out.println("验签结果"+checkSignResult);
            if (!checkSignResult){
                errorDesc="身份认证请求1.0 平台 响应验签出错";
            }
        }

        /**
		 * 组装认证申请返回包
		 */
        if (applicationReturn != null) {
            businessSerialNumberByte = Arrays.copyOfRange(applicationReturn, 24, 33);
            preAuthResponse.setBusinessSerialNumber(FormatBytesUtil.bytesToHexString(businessSerialNumberByte));
            randomBytesLength = FormatBytesUtil.getShortFromBytes(Arrays.copyOfRange(applicationReturn, 33, 35)); // 获取随机数长度
            randomData = FormatBytesUtil
                    .bytesToHexString(Arrays.copyOfRange(applicationReturn, 35, 35 + randomBytesLength)); // 获取随机数数据
            preAuthResponse.setRandomNumber(randomData);
            preAuthResponse.setSuccess(true);
            errorDesc="";

        }
        if (applicationReturn == null) {
            preAuthResponse.setSuccess(false);
            errorDesc="业务流水号获取失败，请重新获取";
        }
        preAuthResponse.setTimeStamp(preAuthBean.getTimeStamp());
        preAuthResponse.setErrorDesc(errorDesc);
        preAuthResponse.setCustomerNumber(preAuthBean.getCustomerNumber());
        preAuthResponse.setAppName(preAuthBean.getAppName());
        String responseStr=JSONObject.fromObject(preAuthResponse).toString();
        return responseStr;
    }
}
