package com.fri.http.legalchapter.rzfw.auth;

import com.fri.http.legalchapter.rzfw.data.*;
import com.fri.http.legalchapter.rzfw.exception.EncryptionException;
import com.fri.http.legalchapter.rzfw.net.SocketSender;
import com.fri.http.legalchapter.rzfw.utils.Constans;
import com.fri.http.legalchapter.rzfw.utils.FormatBytesUtil;
import net.sf.json.JSONObject;
import sun.misc.BASE64Decoder;

import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by GavinLi 李晓阳 on 2017/12/24.
 */
public class AuthRequest {
    /**
     * businessSerialNumberByte 流水号 byte[] 类型
     * customerId 客户号 8字节 共用
     * terminalId 终端号 2字节 共用     1.0写死   2.0用应用名称 FormatBytesUtil.hexStringToBytes(preAuthBean.getAppName());
     * certificateNumber 证书标识 or 业务站点
     * authApplyRetainData 客户端传入认证保留数据String
     * authRetainTwoDataDesign 传入1.0 认证平台认证保留 2项数据 ( 已加密)
     * authRetainFourDataDesign 传入1.0 认证平台认证保留 4项数据 ( 已加密)
     * idcardAuthDataBase64DecodeAfter 传入1.0 认证平台认证ID验证数据( 已加密)
     * pictureAuthDataBase64DecodeAfter 传入1.0 认证平台认证人像 ( 已加密)
     * authCodeDataBase64DecodeAfter 传入1.0 认证平台认证认证码数据 ( 已加密)
     * mode 认证模式
     * authDate 根据模式组合成的数据
     * version 协议版本 2字节 0x0001 共用
     * commandType 命令类型 1字节 0x00(请求)0x01(应答) 共用
     * command 命令 1字节 0x00(身份认证申请)0x01(身份认证) 修改值的内容为1
     * commandContent 命令内容 1字节 保留 0x00 共用
     * operation  操作 1字节 保留 0x00 共用
     * auSPData  身份认证  二包 Socket数据报文
     * authReturn  身份认证 1.0 平台二包 响应结果
     * checkSignResult 验签是否正确
     */
    private byte[] businessSerialNumberByte;
    private byte[] customerId;
    private byte[] terminalId;
    private byte[] timeStamp;
    private byte[] certificateNumber;
    private AuthApplyRetainDataBean authApplyRetainData ;
    private byte[] authRetainTwoDataDesign=null;
    private byte[] authRetainFourDataDesign=null;
    private byte[] idcardAuthDataBase64DecodeAfter=null;
    private byte[] pictureAuthDataBase64DecodeAfter=null;
    private byte[] authCodeDataBase64DecodeAfter=null;
    private byte[] authDataBase64Decode=null;
    private String mode;
    private byte[] authDate=null;
    private byte[] version=new byte[]{0, 1};
    private byte[] commandType = new byte[]{0};
    private byte[] command = new byte[]{1};
    private byte[] commandContent = new byte[]{0};
    private byte[] operation = new byte[]{0};
    private byte[] auSPData;
    private byte[] authReturn;
    private boolean checkSignResult=false;
    private Boolean success=false;
    private String errorDesc="UnKnow Error";
    private byte[] customerId_Length=new byte[] { 1, 2, 3, 4, 5, 6, 7, 8 };
    private byte[] timeStamp_Length=new byte[] { 1, 2,3,4};
    AuthResponse authResponse=new AuthResponse();

    public String authRequestExecute(String authRequestJson){
        JSONObject jsonObject = JSONObject.fromObject(authRequestJson);
        Map config = new HashMap();
        config.put("authApplyRetainData", AuthApplyRetainDataBean.class);
        AuthRequestBean authRequestBean= (AuthRequestBean) JSONObject.toBean(jsonObject, AuthRequestBean.class, config);
        customerId=FormatBytesUtil.getBytesFromString(customerId_Length,authRequestBean.getCustomNumber());
        terminalId=Constans.terminalId;
        timeStamp=FormatBytesUtil.getBytesFromString(timeStamp_Length,authRequestBean.getTimeStamp());
        businessSerialNumberByte=FormatBytesUtil.hexStringToBytes(authRequestBean.getBusinessSerialNumber());
        certificateNumber=Constans.certificateNumber;
        authApplyRetainData = authRequestBean.getAuthApplyRetainData();
        mode=authRequestBean.getAuthMode();

        switch (mode){
            case "0x42":
                getauthRetainTwoDataDesign(authApplyRetainData);
                byte[] identityModel_0X42 = new byte[] { 0x42 };
                pictureAuthDataBase64DecodeAfter=dataProcessingBase64Decode("人像",authRequestBean.getPhotoData());
                AuthenticationData au_0X42 = new AuthenticationData(customerId, terminalId,timeStamp, businessSerialNumberByte, identityModel_0X42,
                        null, null, pictureAuthDataBase64DecodeAfter, authRetainTwoDataDesign, certificateNumber);
                authDate = au_0X42.getAuthenticationData();
                break;
            case "0x40":
                getauthRetainTwoDataDesign(authApplyRetainData);
                byte[] identityModel_0X40 = new byte[] { 0x40 };
                AuthenticationData au_0X40 = new AuthenticationData(customerId, terminalId, timeStamp,businessSerialNumberByte, identityModel_0X40,
                        null, null, null, authRetainTwoDataDesign, certificateNumber);
                authDate = au_0X40.getAuthenticationData();
                break;
            case "0x0f":
                byte[] identityModel_0X0F = new byte[] { 0x0f };
                idcardAuthDataBase64DecodeAfter=dataProcessingBase64Decode("ID",authRequestBean.getIdcardAuthData());
                authCodeDataBase64DecodeAfter=dataProcessingBase64Decode("认证码",authRequestBean.getAuthCode());
                pictureAuthDataBase64DecodeAfter=dataProcessingBase64Decode("人像",authRequestBean.getPhotoData());
                AuthenticationData au_0X0F = new AuthenticationData(customerId, terminalId,timeStamp, businessSerialNumberByte, identityModel_0X0F,
                        idcardAuthDataBase64DecodeAfter, authCodeDataBase64DecodeAfter, pictureAuthDataBase64DecodeAfter, null, certificateNumber);
                authDate = au_0X0F.getAuthenticationData();
                break;
            case "0x4f":
                getauthRetainTwoDataDesign(authApplyRetainData);
                byte[] identityModel_0X4F = new byte[] { 0x4f };
                idcardAuthDataBase64DecodeAfter=dataProcessingBase64Decode("ID",authRequestBean.getIdcardAuthData());
                authCodeDataBase64DecodeAfter=dataProcessingBase64Decode("认证码",authRequestBean.getAuthCode());
                pictureAuthDataBase64DecodeAfter=dataProcessingBase64Decode("人像",authRequestBean.getPhotoData());
                AuthenticationData au_0X4F = new AuthenticationData(customerId, terminalId, timeStamp,businessSerialNumberByte, identityModel_0X4F,
                        idcardAuthDataBase64DecodeAfter, authCodeDataBase64DecodeAfter, pictureAuthDataBase64DecodeAfter, authRetainTwoDataDesign, certificateNumber);
                authDate = au_0X4F.getAuthenticationData();
                break;
            case "0x13":
                getauthRetainFourDataDesign(authApplyRetainData);
                byte[] identityModel_0X13 = new byte[] { 0x13 };
                idcardAuthDataBase64DecodeAfter=dataProcessingBase64Decode("ID",authRequestBean.getIdcardAuthData());
                pictureAuthDataBase64DecodeAfter=dataProcessingBase64Decode("人像",authRequestBean.getPhotoData());
                AuthenticationData au_0X13 = new AuthenticationData(customerId, terminalId,timeStamp, businessSerialNumberByte, identityModel_0X13,
                        idcardAuthDataBase64DecodeAfter,null, pictureAuthDataBase64DecodeAfter, authRetainFourDataDesign, certificateNumber);
                authDate = au_0X13.getAuthenticationData();
                break;
            case "0x1F":
                getauthRetainFourDataDesign(authApplyRetainData);
                byte[] identityModel_0X1F = new byte[] { 0x1f };
                idcardAuthDataBase64DecodeAfter=dataProcessingBase64Decode("ID",authRequestBean.getIdcardAuthData());
                authCodeDataBase64DecodeAfter=dataProcessingBase64Decode("认证码",authRequestBean.getAuthCode());
                pictureAuthDataBase64DecodeAfter=dataProcessingBase64Decode("人像",authRequestBean.getPhotoData());
                AuthenticationData au_0X1F = new AuthenticationData(customerId, terminalId,timeStamp, businessSerialNumberByte, identityModel_0X1F,
                        idcardAuthDataBase64DecodeAfter, authCodeDataBase64DecodeAfter, pictureAuthDataBase64DecodeAfter, authRetainFourDataDesign, certificateNumber);
                authDate = au_0X1F.getAuthenticationData();
                break;
        }
        SocketPackage auSP = new SocketPackage(version, commandType, command, commandContent, operation, authDate);
        auSPData = auSP.getSocketPackage();
        authReturn = SocketSender.sendSocketPackage(auSPData);
        checkSignResult=DataSignature.signCheck(authReturn);
        if (!checkSignResult){
            errorDesc="身份认证1.0 平台二包 响应验签出错";
        }else
            if(authReturn != null) {
            String authResult = new String(Arrays.copyOfRange(authReturn, 33, 37));
            authResponse.setAuthResult(authResult);
            success=true;
            errorDesc="";
        }
        authResponse.setAppName(authRequestBean.getAppName());
        authResponse.setCustomerNumber(authRequestBean.getCustomNumber());
        authResponse.setTimeStamp(authRequestBean.getTimeStamp());
        authResponse.setBusinessSerialNumber(authRequestBean.getBusinessSerialNumber());
        authResponse.setSuccess(success);
        authResponse.setErrorDesc(errorDesc);
        String authResStr=JSONObject.fromObject(authResponse).toString();
        return authResStr;
    }

    public void getauthRetainTwoDataDesign(AuthApplyRetainDataBean retainDate){
      try {
          String name=retainDate.getName();
          String idNumber=retainDate.getIdNumber();
          authRetainTwoDataDesign = new ReservationData(name, idNumber, null, null).getReservasionBytes();
      }catch (EncryptionException e){
          success=false;
          errorDesc="2项保留数据加密失败,检查保留数据";
          e.printStackTrace();
      }

    }

    public void getauthRetainFourDataDesign(AuthApplyRetainDataBean retainDate){
        try{
            String name=retainDate.getName();
            String idNumber=retainDate.getIdNumber();
            String enableDate=retainDate.getEnableDate();
            String expireDate=retainDate.getExpireDate();
            authRetainFourDataDesign = new ReservationData(name, idNumber, enableDate, expireDate)
                    .getReservasionBytes();
        }catch (EncryptionException e){
            success=false;
            errorDesc="4项保留数据加密失败,检查保留数据";
            e.printStackTrace();
        }
    }

    private byte[] excuteBase64Decode(String encodeData) throws IOException {
       byte[] originalData= new BASE64Decoder().decodeBuffer(encodeData);
       return originalData;
    }
    private byte[] dataProcessingBase64Decode(String dataName,String data ){
        switch (dataName){
            case "ID":
                authDataBase64Decode=FormatBytesUtil.hexStringToBytes(data);
                break;
            case "认证码":
                authDataBase64Decode=FormatBytesUtil.hexStringToBytes(data);
                break;
            case "人像":
                try {
                    authDataBase64Decode = excuteBase64Decode(data);
                } catch (IOException e) {
                    errorDesc="人像编码解析出错";
                    success=false;
                    e.printStackTrace();
                }
                break;
        }
        return authDataBase64Decode;
    }
}
